from nixietune.target.target import Target
from nixietune.target.cosine import CosineSimilarityTarget
from nixietune.target.contrastive import ContrastiveTarget
from nixietune.target.infonce import InfoNCETarget
from nixietune.target.triplet import TripletTarget
